<?php
error_reporting(E_ALL);

include 'config.php';
class pms
{
    public $pmsRoot = '';

    /**
     * The construct function.
     * 
     * @access public
     * @return void
     */
    public function __construct()
    {
        $this->setConfig();
        $this->setPmsRoot();
        $this->getSites();
    }

    /**
     * Set the config.
     * 
     * @access public
     * @return void
     */
    public function setConfig()
    {
        global $config;
        $this->config = $config;
    }

    /**
     * Connect to database.
     * 
     * @access public
     * @return void
     */
    public function connectDB()
    {
        try
        {
            $dbh = new PDO("mysql:host={$this->config->db->host}", $this->config->db->user, $this->config->db->password);
            $dbh->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_OBJ);
            $dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
            $dbh->exec("SET NAMES utf8");
            $dbh->exec("SET @@sql_mode= ''");
            $this->dbh = $dbh;
        }
        catch(PDOException $e)
        {
            echo 'Connection failed: ' . $e->getMessage() . "\n";
            die("connect to db failed.\n");
        }
    }

    /**
     * Set the root path of pms.
     * 
     * @access public
     * @return void
     */
    public function setPmsRoot()
    {
        $this->pmsRoot = dirname(dirname(dirname(__FILE__))) . '/';
    }

    /**
     * Get sites on this node.
     * 
     * @access public
     * @return array
     */
    public function getSites()
    {
        $sites = array();
        $this->connectDB();
        $databases = $this->dbh->query("SHOW DATABASES");
        foreach($databases as $row)
        {
            $database = $row->Database;
            if(strpos($database,'pms_') === 0)
            {
                $tables = $this->dbh->query("SHOW TABLES FROM $database");
                if(!$tables->rowCount())
                {
                    /* Drop empty database */
                    $this->dbh->exec("DROP DATABASE `$database`");
                    continue;
                }
                $siteName = str_replace('pms_', '', $database);
                if(!empty($siteName)) $sites[] = $siteName;
            }
        }
        $this->sites = $sites;
        return $sites;
    }

    /**
     * Get the url of a site.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function getSiteURL($subDomain)
    {
        return 'http://' . $subDomain . '.' . $this->config->pms->domain . '/';
    }

    /**
     * Get the db name of a site.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function getSiteDB($subDomain)
    {
        return "pms_$subDomain";
    }

    /**
     * Init the backup directory of a domain.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function initBackDir($subDomain)
    {
        $dir = $this->config->pms->backupBasePath . $subDomain;
        if(!is_dir($dir)) mkdir($dir, 0777, true);
        return $dir;
    }

    /**
     * Get the attatch directory of a domain.
     * 
     * @param   string $subDomain 
     * @access  public
     * @return  string
     */
    public function getAttatchDir($subDomain)
    {
        return $this->pmsRoot . "data/upload/" . $subDomain{0} . "/$subDomain/";
    }

    /**
     * Get the svn directory of a domain. 
     * 
     * @param  string $subDomain 
     * @access public
     * @return string
     */
    public function getSVNDir($subDomain)
    {
        return $this->config->pms->svnBasePath . $subDomain . '/'; 
    }

    /**
     * Get the git directory of a domain. 
     * 
     * @param  string $subDomain 
     * @access public
     * @return string
     */
    public function getGitDir($subDomain)
    {
        return $this->config->pms->gitBasePath . 'git/' . $subDomain . '/'; 
    }

    /**
     * Daily remind 
     * 
     * @access public
     * @return void
     */
    public function dailyRemind()
    {
        $sites = $this->getSites(); 
        foreach($sites as $site)
        {
            $siteDB  = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");
            $company = $this->dbh->query('select * from zt_company where id=1')->fetch();
            $siteURL = $this->getSiteURL($site);
            $remindCMD = "cd {$this->pmsRoot}bin/;php ./ztcli {$siteURL}report-remind.html\n";
            echo `$remindCMD`; 
        }
    }

    /**
     * Backup database and files.
     * 
     * @access public
     * @return void
     */
    public function backup()
    {
        echo `sudo chmod -R 777 {$this->config->pms->backupBasePath}`;

        $sites = $this->getSites();
        foreach($sites as $site)
        {
            $backDir = $this->initBackDir($site);

            /* Delete old files. */
            `find {$backDir}/db   -mtime +5 | xargs rm`;
            `find {$backDir}/file -mtime +5 | xargs rm`;
            `find {$backDir}/svn  -mtime +5 | xargs rm`;
            `find {$backDir}/git  -mtime +5 | xargs rm`;
            `rmdir {$backDir}/db/* ; rmdir {$backDir}/file/* ; rmdir {$backDir}/svn/* ; rmdir {$backDir}/git/*`;

            $this->execBackup($site, $backDir);
        }
    }

    public function execBackup($site, $backDir)
    {
        $backCMD = '';
        $month   = date('Ym');
        $date    = date('Ymd');

        if(!is_dir("$backDir/db/$month"))  mkdir("$backDir/db/$month",   0777, true);
        if(!is_dir("$backDir/file/$month"))mkdir("$backDir/file/$month", 0777, true);
        if(!is_dir("$backDir/svn/$month")) mkdir("$backDir/svn/$month",  0777, true);
        if(!is_dir("$backDir/git/$month")) mkdir("$backDir/git/$month",  0777, true);

        /* delete old file. */
        if(file_exists("$backDir/db/$month/$date.db.bz2"))     $backCMD .= "rm -f $backDir/db/$month/$date.db.bz2\n";
        if(file_exists("$backDir/file/$month/$date.file.bz2")) $backCMD .= "rm -f $backDir/file/$month/$date.file.bz2\n";
        if(file_exists("$backDir/svn/$month/$date.svn.bz2"))   $backCMD .= "rm -f $backDir/svn/$month/$date.svn.bz2\n";
        if(file_exists("$backDir/git/$month/$date.git.bz2"))   $backCMD .= "rm -f $backDir/git/$month/$date.git.bz2\n";

        $siteDB   = $this->getSiteDB($site);
        $dbFile   = "$backDir/db/$month/$site.sql";
        $backCMD .= "mysqldump $siteDB -u {$this->config->db->user} -p{$this->config->db->password} > $dbFile\n";
        $backCMD .= "cd $backDir/db/$month; tar cjvf $backDir/db/$month/$date.db.bz2 *.sql\n";
        $backCMD .= "rm -fr $dbFile\n";

        $attatchDir = $this->getAttatchDir($site);
        if(is_dir($attatchDir)) $backCMD .= "cd $attatchDir; sudo tar cjvf $backDir/file/$month/$date.file.bz2 *\n";

        $svnDir = $this->getSVNDir($site);
        if(is_dir($svnDir)) $backCMD .= "cd $svnDir; sudo tar cjvf $backDir/svn/$month/$date.svn.bz2 *\n";

        $gitDir = $this->getGitDir($site);
        if(is_dir($gitDir)) $backCMD .= "cd $gitDir; sudo tar cjvf $backDir/git/$month/$date.git.bz2 *\n";

        echo date('Y-m-d H:i:s') . " backuping $site\n";
        echo `$backCMD`;
        echo "--- $site end ---\n";
    }

    /**
     * restart apache. 
     * 
     * @access public
     * @return void
     */
    public function repoApacheRestart()
    {
        $restartFile = $this->config->pms->repoBasePath . "apacherestart";
        if(file_exists($restartFile)) 
        {
            exec("sudo apache2ctl -t", $output, $return); 
            if($return == 0)
            {
                `sudo apache2ctl graceful`;
                `sudo rm {$this->config->pms->repoBasePath}apacherestart`;
            }
        }
    }

    /**
     * Update burndown chart.
     * 
     * @access public
     * @return void
     */
    public function updateBurn()
    {
        $sites = $this->getSites();
        foreach($sites as $site)
        {
            $siteURL = $this->getSiteURL($site);
            $updateCMD = "cd {$this->pmsRoot}bin/;php ./ztcli {$siteURL}project-computeburn.html\n";
            echo date('Y-m-d H:i:s') . " updating $siteURL\n";
            echo `$updateCMD`;
        }
    }

    /**
     * Upgrade to latest version.
     * 
     * @access public
     * @return void
     */
    public function upgrade($fromVersion, $toVersion)
    {
        $this->connectDB();
        $sites = $this->getSites();
        $sqls  = $this->getUpgradeSQL($fromVersion);
        
        foreach($sites as $site)
        {
            $siteDB = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");
            echo "Check Consistency For {$site}\n";
            $siteURL = $this->getSiteURL($site);
            $upgradeCMD = "cd {$this->pmsRoot}bin/;php ./ztcli {$siteURL}api-getModel-upgrade-checkConsistency.html;\n";
            $result = `$upgradeCMD`; 
            $result = json_decode($result);
            $data   = !empty($result->data) ? json_decode($result->data) : '';
            if(!empty($data))
            {
                try{$this->dbh->exec($data); }catch(PDOException $e){die($e->getMessage());}
            }
        }

        foreach($sites as $site)
        {
            echo date('Y-m-d H:i:s') . " upgrading $site\n";

            $siteDB = $this->getSiteDB($site);
            $this->dbh->exec("USE $siteDB");
            foreach($sqls as $sql) try{ $this->dbh->exec($sql); }catch(PDOException $e){echo $e->getMessage() . "\n";}

            $siteURL = $this->getSiteURL($site);
            $upgradeCMD  = '';
            $upgradeCMD .= "cd {$this->pmsRoot}bin/;php ./ztcli {$siteURL}api-getModel-upgrade-changeTaskParentValue.html;\n";
            echo `$upgradeCMD`; 

            $this->updateVersion($toVersion);
            $this->updateSN($site);
            echo " finished \n";
        }
    }

    /**
     * Update the version.
     * 
     * @access public
     * @return void
     */
    public function updateVersion($version)
    {
        try{ $this->dbh->exec("UPDATE zt_config SET value='$version' WHERE `key` = 'version'"); }catch(PDOException $e){echo $e->getMessage() . "\n";}
    }

    /**
     * Update the sn.
     * 
     * @access public
     * @return void
     */
    public function updateSN($site)
    {
        $sn = md5(str_shuffle(md5($site . mt_rand(0, 99999999) . microtime())) . microtime());
        try{ $this->dbh->exec("UPDATE zt_config SET value='$sn' WHERE `key` = 'sn' AND value = ''"); }catch(PDOException $e){echo $e->getMessage() . "\n";}
    }

    /**
     * Request by socket. 
     * 
     * @param  int    $remote_server 
     * @param  int    $remote_path 
     * @param  int    $post_string 
     * @param  int    $port 
     * @param  int    $timeout 
     * @access public
     * @return void
     */
    function requestBySocket($remote_server,$remote_path,$post_string,$port = 443,$timeout = 30)
    {
        $socket = fsockopen('ssl://' . $remote_server,$port,$errno,$errstr,$timeout);
        if (!$socket) die("$errstr($errno)");

        fwrite($socket,"POST $remote_path HTTP/1.0\r\n");
        fwrite($socket,"User-Agent: Socket Example\r\n");
        fwrite($socket,"HOST: $remote_server\r\n");
        fwrite($socket,"Content-type: application/x-www-form-urlencoded\r\n");
        fwrite($socket,"Content-length: ". strval(strlen($post_string)+8). "\r\n");
        fwrite($socket,"Accept:*/*\r\n");
        fwrite($socket,"\r\n");
        fwrite($socket,"post=$post_string\r\n");
        fwrite($socket,"\r\n");

        $header = "";
        while ($str = trim(fgets($socket,4096))) {
            $header.=$str;
        }

        $data = "";
        while (!feof($socket)) {
            $data .= fgets($socket,4096);
        }

        return $data;
    }

    /**
     * Svn sync use svn hook.
     * 
     * @param  string    $repos 
     * @param  int       $rev 
     * @access public
     * @return void
     */
    public function svnsync($repos, $rev)
    {
        $svnPath = str_replace($this->config->pms->svnBasePath, '', $repos);
        $domain  = substr($svnPath, 0, strpos($svnPath, '/'));

        $siteURL = $this->getSiteURL($domain);
        $repos   = urlencode($repos);
        $cmd     = "cd {$this->pmsRoot}bin/;php ./ztcli {$siteURL}svn-run-$repos-$rev.html\n";
        echo `$cmd`;
    }

    public function gitsync($repos)
    {
        $gitPath = str_replace($this->config->pms->gitBasePath . 'git/', '', $repos);
        $domain  = substr($gitPath, 0, strpos($gitPath, '/'));

        $siteURL = $this->getSiteURL($domain);
        $repos   = urlencode($repos);
        $cmd     = "cd {$this->pmsRoot}bin/;php ./ztcli {$siteURL}git-run-$repos.html\n";
        echo `$cmd`;
    }

    public function asyncWebhookSend()
    {
        $sites = $this->getSites();
        foreach($sites as $site)
        {
            echo date('Y-m-d H:i:s') . " Send Webhook for $site\n";

            $siteURL = $this->getSiteURL($site);
            $command = "cd {$this->pmsRoot}bin/; php ./ztcli {$siteURL}webhook-asyncSend.html;";
            echo `$command`; 
        }
    }
}
